#!/bin/sh
# =============================================== #
# enter.sh v1.0; job79                            #
# Enter into an existing or new dev container and #
# automatically handle dev container updates.     #
# =============================================== #
set -eu
log() { printf '\e[%sm%s\e[0m %s\n' "${3:-36}" "${2:-○}" "$1"; }
arg() { echo -n " $@"; }

# run_args returns the arguments required for the podman run
# command.
run_args() {
  arg "--name $name"

  # Disable some security settings to make it possible to
  # mount host directories without problems.
  arg "--security-opt label=disable" # disable labeling so mounts don't need to be labeled.
  arg "--userns=keep-id"             # required for ~/.ssh which is usually 700.

  # Use host networking.
  arg "--net=host"

  # Mount the wayland socket. Required to get the system
  # clipbard (wl-copy) working.
  [ -e "/run/user/$UID/wayland-0" ] && arg "-v /run/user/$UID/wayland-0:/run/user/1000/wayland-0"

  # Mount the ssh socket, directory and the git config
  # directory. This gets the host ssh and git configuration
  # working inside the container.
  [ -e "$SSH_AUTH_SOCK" ] && arg "-v $SSH_AUTH_SOCK:/run/user/1000/ssh-auth-sock"
  [ -d "$HOME/.ssh" ] && arg "-v $HOME/.ssh:/home/user/.ssh"
  [ -d "$HOME/.config/git" ] && arg "-v $HOME/.config/git:/home/user/.config/git"

  # Mount host directories with programming projects.
  [ -d "$HOME/Code" ] && arg "-v $HOME/Code:/home/user/Code"
  [ -d "$HOME/.config/devcontainer" ] && arg "-v $HOME/.config/devcontainer:/home/user/.dev"

  # Add volumes for .local and .cache so these survive
  # container restarts.
  arg "-v $name-local:/home/user/.local"
  arg "-v $name-cache:/home/user/.cache"
}

### MAIN ###
name="dev"
image="git.plabble.org/job79/dev:latest"
fetch=false

while test $# -gt 0; do
  case "$1" in
  --image | -i) shift && fetch=true && image="$1" ;;
  --fetch | -f) fetch=true ;;
  -*) log "unknown argument '$1'" 'x' 31 ;;
  *) name="$1" ;;
  esac
  shift
done

if [ "$fetch" = true ] || [ "$(podman container inspect "$name" -f {{.State.Running}} 2>&1)" != 'true' ]; then
  log "fetching devcontainer updates..."
  if [ "$(podman pull -q "$image" 2>&1)" != "$(podman container inspect "$name" -f {{.Image}} 2>&1)" ]; then
    log "container image downloaded" '✓' 32
    podman container rm -f -t 1 "$name" 1>/dev/null
    podman run -td $(run_args) "$image"
  else
    log "no updates available" '✓' 32
  fi
fi

podman start "$name" 1>/dev/null
podman exec --detach-keys "" -it "$name" bash -l
